import "core-js/modules/es6.array.iterator";
import "core-js/modules/es6.object.keys";
import "core-js/modules/web.dom.iterable";
import "core-js/modules/es6.promise";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";

/**
 * @license
 * Copyright © 2017-2018 Moov Corporation.  All rights reserved.
 */

/**
 * Method for checking script tag in <head>
 * @param {String} src path to script
 */
function isScriptInjected(src) {
  if (typeof document === 'undefined') {
    return false;
  }

  var injectedScripts = _toConsumableArray(document.querySelectorAll("script[src^=\"".concat(src, "\"]")));

  return injectedScripts.length > 0;
}
/**
 * Method for checking script tag in <head>
 * @param {String} code script code
 */


function isCodeInjected(code) {
  if (typeof document === 'undefined') {
    return false;
  }

  var injectedCodes = _toConsumableArray(document.head.getElementsByTagName('script')).filter(function (el) {
    return el.innerHTML === code;
  });

  return injectedCodes.length > 0;
}
/**
 * Method for injecting script tag in <head>
 * @param {String} src path to script
 * @param {Object} [params] Optional params
 * @param {Object} [params.attrs={}] Optional attributes to add to element
 * @param {Boolean} [params.once=true] If true then script will be injected only once
 * @returns {Promise} Promise returns on script's onload event
 */


export function injectScriptInHead(src) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$attrs = _ref.attrs,
      attrs = _ref$attrs === void 0 ? {} : _ref$attrs,
      _ref$once = _ref.once,
      once = _ref$once === void 0 ? true : _ref$once;

  if (once && isScriptInjected(src)) {
    return Promise.resolve();
  }

  if (typeof document === 'undefined') {
    return Promise.resolve();
  }

  var promiseResponse;
  var scriptHeadElement = document.createElement('script');
  scriptHeadElement.src = src;
  Object.keys(attrs).forEach(function (key) {
    scriptHeadElement.setAttribute(key, attrs[key]);
  });
  promiseResponse = new Promise(function (resolve, reject) {
    scriptHeadElement.onload = resolve;
    scriptHeadElement.onerror = reject;
  });
  document.head.appendChild(scriptHeadElement);
  return promiseResponse;
}
/**
 * Method for injecting target code in <head>
 * @param {String} code script code
 * @param {Object} [params] optional params
 * @param {Boolean} [params.once=false] If true then script will be injected only once
 */

export function injectCodeInHead(code) {
  var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref2$once = _ref2.once,
      once = _ref2$once === void 0 ? true : _ref2$once;

  if (once && isCodeInjected(code)) {
    return;
  }

  if (typeof document === 'undefined') {
    return;
  }

  var scriptHeadElement = document.createElement('script');
  scriptHeadElement.appendChild(document.createTextNode(code));
  document.head.appendChild(scriptHeadElement);
}
/**
 * Method for removing script in <head>
 * @param {String} src path to script
 */

export function removeScriptInHead(src) {
  ;

  _toConsumableArray(document.querySelectorAll("script[src^=\"".concat(src, "\"]"))).forEach(function (el) {
    return el.parentNode.removeChild(el);
  });
}
/**
 * Method for removing code  in <head>
 * @param {String} code script code
 */

export function removeCodeInHead(code) {
  ;

  _toConsumableArray(document.head.getElementsByTagName('script')).filter(function (el) {
    return el.innerHTML === code;
  }).forEach(function (el) {
    return el.parentNode.removeChild(el);
  });
}
var scriptInjector = {
  injectScriptInHead: injectScriptInHead,
  injectCodeInHead: injectCodeInHead,
  removeScriptInHead: removeScriptInHead,
  removeCodeInHead: removeCodeInHead
};
export default scriptInjector;
//# sourceMappingURL=scriptInjector.js.map